@@ -6,15 +6,19 @@ Main function is ``get_producer()``.
 
 import json
 import logging
+import threading
+import time
+import weakref
 from functools import lru_cache
 from typing import Any, List, Optional
 
+from django.conf import settings
 from django.dispatch import receiver
 from django.test.signals import setting_changed
 from openedx_events.event_bus.avro.serializer import AvroSignalSerializer
 from openedx_events.tooling import OpenEdxPublicSignal
 
-from .config import get_schema_registry_client, load_common_settings
+from .config import get_full_topic, get_schema_registry_client, load_common_settings
 
 logger = logging.getLogger(__name__)
 
@@ -167,6 +171,13 @@ class EventProducerKafka():
     def __init__(self, producer):
         self.producer = producer
 
+        threading.Thread(
+            target=poll_indefinitely,
+            name="kafka-producer-poll",
+            args=(weakref.ref(self),),  # allow GC but also thread auto-stop (important for tests!)
+            daemon=True,  # don't block shutdown
+        ).start()
+
     def send(
             self, *, signal: OpenEdxPublicSignal, topic: str, event_key_field: str, event_data: dict,
     ) -> None:
@@ -187,18 +198,18 @@ class EventProducerKafka():
         key_bytes = key_serializer(event_key, SerializationContext(topic, MessageField.KEY, headers))
         value_bytes = value_serializer(event_data, SerializationContext(topic, MessageField.VALUE, headers))
 
+        full_topic = get_full_topic(topic)
+
         self.producer.produce(
-            topic, key=key_bytes, value=value_bytes, headers=headers, on_delivery=on_event_deliver,
+            full_topic, key=key_bytes, value=value_bytes, headers=headers, on_delivery=on_event_deliver,
         )
 
         # Opportunistically ensure any pending callbacks from recent event-sends are triggered.
-        #
-        # This assumes events come regularly, or that we're not concerned about
-        # high latency between delivery and callback. If those assumptions are
-        # false, we should switch to calling poll(1.0) or similar in a loop on
-        # a separate thread. Or do both.
-        #
-        # Issue: https://github.com/openedx/event-bus-kafka/issues/31
+        # This ensures that we're polling at least as often as we're producing, which is a
+        # reasonable balance. However, if events are infrequent, it doesn't ensure that
+        # callbacks happen in a timely fashion, and the last event emitted before shutdown
+        # would never get a delivery callback. That's why there's also a thread calling
+        # poll(0) on a regular interval (see `poll_indefinitely`).
         self.producer.poll(0)
 
     def prepare_for_shutdown(self):
@@ -210,6 +221,47 @@ class EventProducerKafka():
         self.producer.flush(-1)
 
 
+def poll_indefinitely(api_weakref: EventProducerKafka):
+    """
+    Poll the producer indefinitely to ensure delivery/stats/etc. callbacks are triggered.
+
+    The thread stops automatically once the producer is garbage-collected.
+
+    See ADR for more information:
+    https://github.com/openedx/event-bus-kafka/blob/main/docs/decisions/0007-producer-polling.rst
+    """
+    # The reason we hold a weakref to the whole EventProducerKafka and
+    # not directly to the Producer itself is that you just can't make
+    # a weakref to the latter (perhaps because it's a C object.)
+
+    # .. setting_name: EVENT_BUS_KAFKA_POLL_INTERVAL_SEC
+    # .. setting_default: 1.0
+    # .. setting_description: How frequently to poll the event-bus-kafka producer. This should
+    #   be small enough that there's not too much latency in triggering delivery callbacks once
+    #   a message has been acknowledged, but there's no point in setting it any lower than the
+    #   expected round-trip-time of message delivery and acknowledgement. (100 ms – 5 s is
+    #   probably a reasonable range.)
+    poll_interval_seconds = getattr(settings, 'EVENT_BUS_KAFKA_POLL_INTERVAL_SEC', 1.0)
+    while True:
+        time.sleep(poll_interval_seconds)
+
+        # Temporarily hold a strong ref to the producer API singleton
+        api_object = api_weakref()
+        if api_object is None:
+            return
+
+        try:
+            api_object.producer.poll(0)
+        except BaseException:
+            # If polling is failing, we'll almost certainly find out about it from the poll call
+            # we make when producing an event. The call in this loop could be excessively noisy,
+            # so just debug-log it.
+            logger.debug("Event bus producer polling loop encountered exception (continuing)", exc_info=True)
+        finally:
+            # Get rid of that strong ref again
+            api_object = None
+
+
 # Note: This caching is required, since otherwise the Producer will
 # fall out of scope and be garbage-collected, destroying the
 # outbound-message queue and threads. The use of this cache allows the
